from django.contrib import auth
from django.contrib.auth.models import User
from django.conf import settings
from django.shortcuts import redirect

from urlauth.util import load_key, InvalidKey
from urlauth.models import AuthKey
from urlauth.signals import authkey_processed


class AuthKeyMiddleware(object):
    """
    This middleware can authenticate user with auth key in HTTP request.
    """

    def process_request(self, request):
        key = request.REQUEST.get(settings.URLAUTH_AUTHKEY_NAME)
        
        user = None
        try:
            key = load_key(key)
            if key.uid:
                try:
                    user = User.objects.get(pk=key.uid)
                except User.DoesNotExist:
                    raise InvalidKey('User [pk=%s] does not exist' % user.pk)
        except InvalidKey:
            return

        authkey_processed.send(sender=AuthKey, key=key, request=request, user=user)

        if user and user.is_active:
            user.backend = 'django.contrib.auth.backends.ModelBackend'
            auth.login(request, user)    

        if key.onetime:
            key.delete()

        request.authkey = key



from datetime import datetime

from django.core.management.base import BaseCommand

from urlauth.models import AuthKey

class Command(BaseCommand):
    help = 'Remove expired keys from database'

    def handle(self, *args, **kwargs):
        qs = AuthKey.objects.filter(expired__lt=datetime.now())
        count = qs.count()
        qs.delete()
        print '%d keys deleted' % count



import os
from datetime import datetime
import time
from base64 import b64encode, b64decode
from urllib import urlencode
from cgi import parse_qs

from django.conf import settings

from urlauth.models import AuthKey


class InvalidKey(Exception):
    pass


def wrap_url(url, **kwargs):
    """
    Create new authorization key and append it to the url.
    """

    if not 'uid' in kwargs:
        raise Exception('wrap_url function requires uid name argument')

    key_id = AuthKey.objects.create_key(**kwargs)
    clue = '?' in url and '&' or '?'
    parts = url.rsplit('#', 1)
    if len(parts) > 1:
        url, hash = parts
        hash = '#%s' % hash
    else:
        hash = ''
    url = '%s%s%s=%s%s' % (url, clue, settings.URLAUTH_AUTHKEY_NAME, key_id, hash)
    return url


def load_key(hash):
    """
    Load key record associated with given hash.
    """

    try:
        key = AuthKey.objects.get(pk=hash)
    except AuthKey.DoesNotExist:
        raise InvalidKey('Key does not exist')
    if datetime.now() > key.expired:
        raise InvalidKey('Key is expired')
    return key



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Changing field 'AuthKey.uid'
        db.alter_column('urlauth_authkey', 'uid', self.gf('django.db.models.fields.PositiveIntegerField')(null=True))


    def backwards(self, orm):
        
        # User chose to not deal with backwards NULL issues for 'AuthKey.uid'
        raise RuntimeError("Cannot reverse this migration. 'AuthKey.uid' and its values cannot be restored.")


    models = {
        'urlauth.authkey': {
            'Meta': {'object_name': 'AuthKey'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'data': ('django.db.models.fields.TextField', [], {}),
            'expired': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.CharField', [], {'max_length': '40', 'primary_key': 'True'}),
            'onetime': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'uid': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True'})
        }
    }

    complete_apps = ['urlauth']



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'AuthKey'
        db.create_table('urlauth_authkey', (
            ('id', self.gf('django.db.models.fields.CharField')(max_length=40, primary_key=True)),
            ('uid', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('expired', self.gf('django.db.models.fields.DateTimeField')()),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('onetime', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('data', self.gf('django.db.models.fields.TextField')()),
        ))
        db.send_create_signal('urlauth', ['AuthKey'])


    def backwards(self, orm):
        
        # Deleting model 'AuthKey'
        db.delete_table('urlauth_authkey')


    models = {
        'urlauth.authkey': {
            'Meta': {'object_name': 'AuthKey'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'data': ('django.db.models.fields.TextField', [], {}),
            'expired': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.CharField', [], {'max_length': '40', 'primary_key': 'True'}),
            'onetime': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'uid': ('django.db.models.fields.PositiveIntegerField', [], {})
        }
    }

    complete_apps = ['urlauth']



import os
from setuptools import setup

# Compile the list of packages available, because distutils doesn't have
# an easy way to do this.

packages, data_files = [], []
root_dir = os.path.dirname(__file__)
if root_dir:
    os.chdir(root_dir)

PACKAGE = 'urlauth'

for dirpath, dirnames, filenames in os.walk(PACKAGE):
    for i, dirname in enumerate(dirnames):
        if dirname in ['.', '..']:
            del dirnames[i]
    if '__init__.py' in filenames:
        pkg = dirpath.replace(os.path.sep, '.')
        if os.path.altsep:
            pkg = pkg.replace(os.path.altsep, '.')
        packages.append(pkg)
    elif filenames:
        prefix = dirpath[len(PACKAGE) + 1:] # Strip package directory + path separator
        for f in filenames:
            data_files.append(os.path.join(prefix, f))

setup(
    version = '0.1.5',
    description = 'Django application for user authentication with key in hypertext link',
    author = 'Grigoriy Petukhov',
    author_email = 'lorien@lorien.name',
    url = 'http://bitbucket.org/lorien/django-urlauth/',
    name = 'django-urlauth',

    packages = packages,
    package_data = {'urlauth': data_files},

    license = "BSD",
    keywords = "django application authentication authorization",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Utilities'
    ],
)



try:
    from hashlib import sha1
except ImportError:
    from sha import new as sha1
import time
from datetime import datetime, timedelta

from django.db import models
from django.conf import settings
from django.utils import simplejson
from django.contrib.auth.models import User


class AuthKeyManager(models.Manager):

    def create_key(self, **kwargs):
        """
        Create AuthKey object and return its ID.
        """

        key = AuthKey()
        key.uid = kwargs.pop('uid')
        if 'expired' in kwargs:
            key.expired = kwargs.pop('expired')
        else:
            key.expired = datetime.now() + timedelta(seconds=settings.URLAUTH_AUTHKEY_TIMEOUT)

        if 'onetime' in kwargs:
            key.onetime = kwargs.pop('onetime')

        source = '%s%d%d' % (settings.SECRET_KEY, time.time(), id({}))
        key.id = sha1(source).hexdigest()
        key.data = simplejson.dumps(kwargs)
        key.save()
        return key.id


class AuthKey(models.Model):
    id = models.CharField(max_length=40, primary_key=True)
    uid = models.PositiveIntegerField(null=True)
    expired = models.DateTimeField()
    created = models.DateTimeField(auto_now_add=True)
    onetime = models.BooleanField(blank=True, default=True)
    data = models.TextField()

    objects = AuthKeyManager()

    def __unicode__(self):
        return 'AuthKey #%s' % self.id

    @property
    def extra(self):
        return simplejson.loads(self.data)

    def get_user(self):
        if self.uid:
            return User.objects.get(pk=self.uid)
        else:
            return None
